const hour = 60 * 60

class MemcachedAdaptor {
  constructor ({ client, namespace, lifetime = hour }) {
    this.client = client
    this.namespace = namespace
    this.lifetime = lifetime
  }

  _withNamespace (key) {
    const namespace = this.namespace
    const keyWithNamespace = namespace
      ? [namespace, ...key]
      : key

    return keyWithNamespace.join(':')
  }

  set (key, value) {
    const opts = {
      lifetime: this.lifetime,
    }
    if (Array.isArray(value)) {
      const promises = []
      value.forEach((v, i) => {
        promises.push(this.client.set(this._withNamespace([key[0]].concat(key[1][i])), JSON.stringify(v), opts))
      })
      return Promise.all(promises)
    }
    return this.client.set(this._withNamespace(key), JSON.stringify(value), opts)
  }

  get (key) {
    if (Array.isArray(key[1])) {
      const keys = key[1].map(k => this._withNamespace([key[0], k]));
      return this.client.get(keys).then(r => {
        const rows = {};
        key[1].forEach(k => {
          const ck = this._withNamespace([key[0], k]);
          if (r[ck] && r[ck].value) {
            rows[k] = JSON.parse(r[ck].value);
          }
        });
        return rows;
      });
    } else {
      return this.client.get(this._withNamespace(key)).then(r => JSON.parse(r.value));
    }
  }

  del (key) {
    return this.client.delete(this._withNamespace(key))
  }
}

module.exports = MemcachedAdaptor
